/**
 * BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2016
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
*/

/* BM FileServer API */

function BMFileHostingAPI(aBaseUrl, aAuthKey, aDomainUid) {
    this._logger = Components.classes["@blue-mind.net/logger;1"].getService().wrappedJSObject.getLogger("BMFileHostingAPI: ");
    this._authKey = aAuthKey;
    this._baseUrl = aBaseUrl + "/api";
    this._domainUid = aDomainUid;
}

BMFileHostingAPI.prototype.store = function(aOnLoad, aOnError, aExtraHeader, path, file) {
    let url = this._baseUrl + "/filehosting/" + this._domainUid + "/" + encodeURIComponent(path);
    return this._execute(aOnLoad, aOnError, url, "PUT", aExtraHeader, file);
};

BMFileHostingAPI.prototype.share = function(aOnLoad, aOnError, path, downloadLimit, expirationDate) {
    let url = this._baseUrl + "/filehosting/" + this._domainUid + "/_share";
    url += "?path=" + encodeURIComponent(path);
    url += "&downloadLimit=" + encodeURIComponent(downloadLimit);
    url += "&expirationDate=" + encodeURIComponent(expirationDate);
    return this._execute(aOnLoad, aOnError, url, "GET");
};

BMFileHostingAPI.prototype.unShare = function(aOnLoad, aOnError, uurl) {
    let url = this._baseUrl + "/filehosting/" + this._domainUid + "/" + encodeURIComponent(uurl) + "/unshare";
    return this._execute(aOnLoad, aOnError, url, "DELETE");
};

BMFileHostingAPI.prototype.getConfig = function(aOnLoad, aOnError) {
    let url = this._baseUrl + "/attachment/" + this._domainUid + "/_config";
    return this._execute(aOnLoad, aOnError, url, "GET");
};

BMFileHostingAPI.prototype.detach = function(aOnLoad, aOnError, aExtraHeader, name, file) {
    let url = this._baseUrl + "/attachment/" + this._domainUid + "/" + encodeURIComponent(name) + "/share";
    return this._execute(aOnLoad, aOnError, url, "PUT", aExtraHeader, file);
};

BMFileHostingAPI.prototype._execute = async function(aOnLoad, aOnError,  aUrl, aMethod, aExtraHeader, aData) {
    let hds = [["X-BM-ApiKey", this._authKey]];
    let headers = aExtraHeader ? hds.concat(aExtraHeader) : hds;
    let response = await fetch(aUrl, {
        headers: headers.reduce((res, h) => {
            res[h[0]] = h.slice(1).join("; ");
            return res;
        },{}),
        method: aMethod,
        body: aData
    });
    response.ok ? aOnLoad(response) : aOnError(response);
    return response;
}