/**
 * BEGIN LICENSE
 * Copyright Â© Blue Mind SAS, 2012-2016
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
 */

/**
 * RPC Client
 */
ComputerTelephonyIntegrationClient._rpcClient;

/**
 * URL
 */
ComputerTelephonyIntegrationClient._baseUrl;

/**
 * @type {string} domainUid
 * @private
 */
ComputerTelephonyIntegrationClient.prototype._domainUid;

/**
 * @type {string} userUid
 * @private
 */
ComputerTelephonyIntegrationClient.prototype._userUid;

/**
 * @constructor
 */ 
function ComputerTelephonyIntegrationClient(base, sid , domainUid , userUid ) {
  this._rpcClient = Components.classes["@blue-mind.net/rpcclient;1"].getService().wrappedJSObject;
  this._rpcClient.setAuthKey(sid ? sid : null);
  this._baseUrl = base + '/api/cti/{domainUid}/{userUid}';
  
  this._domainUid = domainUid;
  this._baseUrl = this._baseUrl.replace('{domainUid}',domainUid);
  this._userUid = userUid;
  this._baseUrl = this._baseUrl.replace('{userUid}',userUid);
};

ComputerTelephonyIntegrationClient.prototype.setAuthKey = function(sid) {
    this._rpcClient.setAuthKey(sid);
};

/*
 *
 */
ComputerTelephonyIntegrationClient.prototype.dial = function( number  ) {
  let url = this._baseUrl + "/dial";


  url += "?number=" + encodeURIComponent(number);
  let responseType = "";
  let cmd = this._rpcClient.newCommand(url, "POST", responseType);
  let body = null;
  return this._execute(cmd, body);
};

/*
 *
 */
ComputerTelephonyIntegrationClient.prototype.forward = function( component ,  phoneNumber  ) {
  let url = this._baseUrl + "/forward/{component}";

  url = url.replace("{component}", component);

  url += "?phoneNumber=" + encodeURIComponent(phoneNumber);
  let responseType = "";
  let cmd = this._rpcClient.newCommand(url, "POST", responseType);
  let body = null;
  return this._execute(cmd, body);
};

/*
 *
 */
ComputerTelephonyIntegrationClient.prototype.getStatus = function() {
  let url = this._baseUrl + "/status";


  let responseType = "";
  let cmd = this._rpcClient.newCommand(url, "GET", responseType);
  let body = null;
  return this._execute(cmd, body);
};

/*
 *
 */
ComputerTelephonyIntegrationClient.prototype.getUserEmails = function() {
  let url = this._baseUrl + "/users";


  let responseType = "";
  let cmd = this._rpcClient.newCommand(url, "GET", responseType);
  let body = null;
  return this._execute(cmd, body);
};

/*
 *
 */
ComputerTelephonyIntegrationClient.prototype.setStatus = function( component ,  status  ) {
  let url = this._baseUrl + "/status/{component}";

  url = url.replace("{component}", component);

  let responseType = "";
  let cmd = this._rpcClient.newCommand(url, "POST", responseType);
  let body = null;
  body = status;
  return this._execute(cmd, body);
};

ComputerTelephonyIntegrationClient.prototype._execute = function(cmd, data) {
  cmd.getData = function() {
    if (data != null){
      return JSON.stringify(data);
    } 
    return null;
  };

  cmd.onSuccess = function(xhr) {
    let contentType = xhr.getResponseHeader("content-type");
    if (contentType == "application/json" && xhr.responseText) {
      cmd.resolve(JSON.parse(xhr.responseText));
    } else {
      cmd.resolve(xhr.response);
    }
  };

  cmd.onFailure = function(xhr) {
    let resp = xhr.response;
    if (resp != null) {
        cmd.reject(resp);
    } else {
        cmd.reject(xhr.status);
    }
  };

  let self = this;
  let result = new Promise(function(resolve, reject) {
    try {
      cmd.resolve = resolve;
      cmd.reject = reject;
      self._rpcClient.execute(cmd);
    } catch(e) {
      reject(e);
    }
  });
  return result;
}
