/**
 * BEGIN LICENSE
 * Copyright © Blue Mind SAS, 2012-2020
 *
 * This file is part of BlueMind. BlueMind is a messaging and collaborative
 * solution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU Affero General Public License as
 * published by the Free Software Foundation (version 3 of the License).
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See LICENSE.txt
 * END LICENSE
*/

/* Remote file chooser */

var Services = globalThis.Services ||(!Cu.unload
	? ChromeUtils.importESModule("resource://gre/modules/Services.sys.mjs").Services
	: ChromeUtils.import("resource://gre/modules/Services.jsm").Services);

var { bmUtils } = !Cu.unload
	? ChromeUtils.importESModule("chrome://bm/content/modules/bmUtils.sys.mjs?5.3.2760.0")
	: ChromeUtils.import("chrome://bm/content/modules/bmUtils.jsm");

var gBMRemoteChooser = {
  _logger: Components.classes["@blue-mind.net/logger;1"].getService().wrappedJSObject.getLogger("gBMRemoteChooser: "),
  _callback: null,
  _baseUri: null,
  _notify: {},
  loadWindow: async function () {
    console.trace("loadWindow");
    Services.scriptloader.loadSubScript("chrome://bm/content/notifyTools.js", this._notify, "UTF-8");
    let user = {};
    let pwd = {};
    let srv = {};
    if (bmUtils.getSettings(user, pwd, srv, true, window)) {
      if (!Services.io.offline) {
        this._baseUri = srv.value;
        this._auth(srv.value, user.value, pwd.value);
      }
    } else {
      this._logger.info("not enouth settings");
    }
  },
  _auth: async function (aServer, aLogin, aPassword) {
    let host = aServer.replace("https://", "");
    let cookies = Services.cookies.getCookiesFromHost(host, {});
    for (let cookie of cookies) {
      Services.cookies.remove(cookie.host, cookie.name, cookie.path, {});
    }
    let url = aServer + "/login/native";
    let response = await this._notify.notifyTools.notifyBackground({command: "fetch", url: url});
    if (response.responseText) {
			cookies = Services.cookies.getCookiesFromHost(host, {});
			let bmSession = null;
			for (let cookie of cookies) {
				if (cookie.name == "BMSESSION") {
					bmSession = cookie.value;
				}
			}
			this._login(aServer, aLogin, aPassword, bmSession);
		} else {
			this._logger.error("Fail to login", response.responseText);
		}
  },
  _login: async function (aServer, aLogin, aPassword, aBmSession) {
    let host = aServer.replace("https://", "");
    let url = aServer + "/auth/form";
    let postData = [["login", aLogin],
                    ["password", aPassword],
                    ["priv", "priv"],
                    ["askedUri", "/"],
                    ["csrfToken", aBmSession]];
    let data = {
      server: aServer,
      bmSession: aBmSession,
      cookiesHost: host
    };

    let response = await this._notify.notifyTools.notifyBackground({command: "fetch",
			url: url,
			options: {
				method: "POST",
				params: postData
			}
		});
    if (response.responseText) {
			this._onAuth(response.responseText, data);
		} else {
			//FIXME check response.status for offline open ?
			//this._openWebPages(data.server, data.apps);
			this._logger.error("Fail to login: " + response.responseText);
		}
  },
  _onAuth: function (aResponseText, aData) {
    if (aResponseText.indexOf("<meta name=\"X-BM-Page\" content=\"maintenance\"/>") != -1) {
      this._logger.info("BM server is in maintenance");
      //FIXME what ?
    } else {
      let cookies = Services.cookies.getCookiesFromHost(aData.cookiesHost, {});
      let ssoCookie = null;
      for (let cookie of cookies) {
        if (cookie.name == "BMSID") {
          ssoCookie = cookie.value;
        }
      }
      this._logger.debug("ssoCookie:" + ssoCookie);
      if (!ssoCookie && aResponseText.indexOf("Login failed for user") != -1) {
        bmUtils.promptService.alert(null, bmUtils.getLocalizedString("dialogs.title"),
          bmUtils.getLocalizedString("errors.credentials"));
        bmUtils.removeCredentialStored(aData.server);
        bmUtils.session.password = null;
      } else {
        this._openWebPage();
      }
    }
  },
  _openWebPage: function () {
    this._notify.notifyTools.notifyBackground({command: "openPopup", url: this._baseUri + "/chooser/#"});
  },
}
